@@ -1,29 +1,11 @@
 {-# LANGUAGE CPP #-}
 module Test.Hspec.Core.Compat (
-  getDefaultConcurrentJobs
-, showType
-, showFullType
-, readMaybe
-, lookupEnv
-, module Data.IORef
-
-, module Prelude
-, module Control.Applicative
-, module Control.Monad
-, module Data.Foldable
-, module Data.Traversable
-, module Data.Monoid
-, module Data.List
-
-#if !MIN_VERSION_base(4,6,0)
-, modifyIORef'
-, atomicWriteIORef
-#endif
-, interruptible
+  module Imports
+, module Test.Hspec.Core.Compat
 ) where
 
-import           Control.Applicative
-import           Control.Monad hiding (
+import           Control.Applicative as Imports
+import           Control.Monad as Imports hiding (
     mapM
   , mapM_
   , forM
@@ -32,12 +14,28 @@ import           Control.Monad hiding (
   , sequence
   , sequence_
   )
-import           Data.Foldable
-import           Data.Traversable
-import           Data.Monoid
-import           Data.List (intercalate)
+import           Data.Foldable as Imports
 
-import           Prelude hiding (
+#if MIN_VERSION_base(4,11,0)
+import           Data.Functor as Imports
+#endif
+
+import           Data.Traversable as Imports
+import           Data.Monoid as Imports
+import           Data.List as Imports (
+    stripPrefix
+  , isPrefixOf
+  , isInfixOf
+  , intercalate
+  , inits
+  , tails
+  , sortBy
+#if MIN_VERSION_base(4,8,0)
+  , sortOn
+#endif
+  )
+
+import           Prelude as Imports hiding (
     all
   , and
   , any
@@ -58,25 +56,37 @@ import           Prelude hiding (
   , sequence
   , sequence_
   , sum
+#if !MIN_VERSION_base(4,6,0)
+  , catch
+#endif
   )
 
 import           Data.Typeable (Typeable, typeOf, typeRepTyCon)
+import           Data.IORef as Imports
+
+#if MIN_VERSION_base(4,6,0)
+import           Text.Read as Imports (readMaybe)
+import           System.Environment as Imports (lookupEnv)
+#else
 import           Text.Read
-import           Data.IORef
 import           System.Environment
+import qualified Text.ParserCombinators.ReadP as P
+#endif
+
+#if !MIN_VERSION_base(4,8,0)
+import           Data.Ord (comparing)
+#endif
 
 import           Data.Typeable (tyConModule, tyConName)
 import           Control.Concurrent
 
 #if MIN_VERSION_base(4,9,0)
-import           Control.Exception (interruptible)
+import           Control.Exception as Imports (interruptible)
 #else
 import           GHC.IO
 #endif
 
 #if !MIN_VERSION_base(4,6,0)
-import qualified Text.ParserCombinators.ReadP as P
-
 -- |Strict version of 'modifyIORef'
 modifyIORef' :: IORef a -> (a -> a) -> IO ()
 modifyIORef' ref f = do
@@ -139,3 +149,18 @@ interruptible act = do
     MaskedInterruptible   -> unsafeUnmask act
     MaskedUninterruptible -> act
 #endif
+
+guarded :: Alternative m => (a -> Bool) -> a -> m a
+guarded p a = if p a then pure a else empty
+
+#if !MIN_VERSION_base(4,8,0)
+sortOn :: Ord b => (a -> b) -> [a] -> [a]
+sortOn f =
+  map snd . sortBy (comparing fst) . map (\x -> let y = f x in y `seq` (y, x))
+#endif
+
+#if !MIN_VERSION_base(4,11,0)
+infixl 1 <&>
+(<&>) :: Functor f => f a -> (a -> b) -> f b
+(<&>) = flip (<$>)
+#endif
